//floppy assembly
//floppy.c

#include <stdio.h>
#include "floppy.h"

namespace xatremu
{

FloppyDrive::FloppyDrive(){
};

FloppyDrive::FloppyDrive(char * romName){
#ifdef WITH_CAPS
	m_CAPSImgId = CAPSAddImage();
#endif
	m_ROM = new ROMMgmt();
	m_CPU8040 = new CPU8040();
	m_WD1772 = new WD1772Real();
	m_Serial = new SerialReal();
	if(
#ifdef WITH_CAPS
	(m_CAPSImgId < 0) || 
#endif
		(m_ROM == NULL) || (m_CPU8040 == NULL) || (m_WD1772 == NULL) || (m_Serial == NULL)){
		throw;
	};
	m_Serial->SetCPU(m_CPU8040);
	m_WD1772->SetCPU(m_CPU8040);
	m_CPU8040->SetSerial(m_Serial);
	m_CPU8040->SetWD1772(m_WD1772);
	m_ROM->LoadData(romName);
	m_CPU8040->SetInstrMemPtr(m_ROM->GetDataPtr());
	m_CPU8040->SetROMLen(m_ROM->GetDataLen());
	m_CPU8040->ResetCPU();
	m_Serial->ResetSerial();
	m_WD1772->ResetWD1772();
#ifdef WITH_CAPS
	m_WD1772->SetCapsImgId(m_CAPSImgId);
#endif
};

FloppyDrive::~FloppyDrive(){
	if(m_WD1772 != NULL){
		delete m_WD1772;
		m_WD1772 = NULL;
	};
	if(m_Serial != NULL){
		delete m_Serial;
		m_Serial = NULL;
	};
	if(m_CPU8040 != NULL){
		delete m_CPU8040;
		m_CPU8040 = NULL;
	};
	if(m_ROM != NULL){
		delete m_ROM;
		m_ROM = NULL;
	};
#ifdef WITH_CAPS
	if(m_CAPSImgId >= 0){
		CAPSRemImage(m_CAPSImgId);
	};
#endif
};

void FloppyDrive::SetTo288RPM(bool is288RPM){
	m_WD1772->Choose288RPM(is288RPM);
};

bool FloppyDrive::SetImage(char * imageName){
	return m_WD1772->SetImage(imageName);
};

void FloppyDrive::EjectImage(){
	m_WD1772->EjectImage();
};

void FloppyDrive::SetIntLineAtari(bool intLine){
	m_Serial->SetIntLineAtari(intLine);
};

void FloppyDrive::SerialByteFromAtari(unsigned char myByte){
	m_Serial->SerialByteFromAtari(myByte);
};

bool FloppyDrive::ReadyForNextInByte(){
	return m_Serial->ReadyForNextInByte();
};

unsigned char FloppyDrive::SerialByteToAtari(){
	return m_Serial->SerialByteToAtari();
};

bool FloppyDrive::SerialByteForAtariAvailable(){
	return m_Serial->SerialOutByteComplete();
};

void FloppyDrive::RunCPUOneScanline(){
	m_CPU8040->RunCPUForCycles(534);
};

void FloppyDrive::RunCPUForCycles(unsigned int numOfCycles){
	m_CPU8040->RunCPUForCycles(numOfCycles);
};

void FloppyDrive::SetDriveNumber(unsigned char DriveNo){
	m_CPU8040->SetDriveNumber(DriveNo);
};

void FloppyDrive::SetSIOSpeed(unsigned char Speed){
	m_Serial->SetSIOSpeed(Speed);
};

};
